import java.awt.*;
import java.awt.event.*;  
import java.awt.image.*;  
import java.util.*;
import javax.swing.*;
import javax.imageio.*;
import java.io.*;
import java.nio.ByteBuffer;

//==============================================
public class ImageMemoryInputStream
{
   int remainingBytes = 0;
   
   int bitPos = 0;
   
   int[] pixels = null;
   
   //---------------------------------------------
   public ImageMemoryInputStream(Image img) throws InterruptedException
   {
      int width = img.getWidth(null);
      int height = img.getHeight(null);
      
      remainingBytes = (width * height) * 3 / 8; 
      
      pixels = new int[width * height];
      
      try {
			PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
			pg.grabPixels();
		}
		catch(InterruptedException e) {
			throw e;
		}
   }
   
   //---------------------------------------------
   int read() throws IOException 
   {
      if(pixels == null) throw new IOException("ȼ ۰ ǵ ʾҽϴ.");
      
      if(remainingBytes == 0) return -1;
      
      int oneByte = 0;
       
      for(int i=7; i >= 0; i--) {
  
         int pixelIndex = bitPos / 3;
         
         int rgbPos = (2 - (bitPos % 3)) * 8;

         int oneBit = (pixels[pixelIndex] >> rgbPos) & 1;
         
         if(oneBit == 1) oneByte |= (1 << i);
 
         bitPos++;
      }
      
      remainingBytes--;
      
      return oneByte;
   }
   
   //---------------------------------------------
   int read(byte[] buff) throws IOException
   {
      int i=0;

      for(i=0; i<buff.length; i++) {
         int x = read();
         if(x == -1) return i;
         buff[i] = (byte)x;
      }

      return i;
   }
   
   //---------------------------------------------
   String readString() throws IOException
   {
      int bufflen = readInt();
      
      if(bufflen < 0) throw new IOException("buffer length cannot be negative.  len: " + bufflen);
      if(bufflen > 999999) throw new IOException("buffer length is too long in readStrng.  len: " + bufflen);
      
      byte[] buff = new byte[bufflen];
      read(buff);
      return new String(buff);
   }
   
   //---------------------------------------------
   long readLong() throws IOException
   {
      byte[] buff = new byte[Long.BYTES];
      int i = read(buff);
      if(i != Long.BYTES) throw new IOException("Read Long length error.");
      return bytesToLong(buff);
   }
   
   //---------------------------------------------
   int readInt() throws IOException
   {
      byte[] buff = new byte[Integer.BYTES];
      int i = read(buff);
      if(i != Integer.BYTES) throw new IOException("Read Integer length error.");
      return bytesToInt(buff);
   }
   
   //---------------------------------------------
   private long bytesToLong(byte[] bytes) 
   {
      ByteBuffer buffer = ByteBuffer.allocate(Long.BYTES);
      buffer.put(bytes);
      buffer.flip();//need flip 
      return buffer.getLong();
   }
   
   //---------------------------------------------
   private int bytesToInt(byte[] bytes) 
   {
      ByteBuffer buffer = ByteBuffer.allocate(Integer.BYTES);
      buffer.put(bytes);
      buffer.flip();//need flip 
      return buffer.getInt();
   }
   
   //---------------------------------------------
   void close() throws IOException
   {
      pixels = null;
      remainingBytes = 0;
      bitPos = 0;
   }
   
   //---------------------------------------------
   int available()
   {
      return remainingBytes;
   }
   
   
}