import java.awt.*;
import java.awt.event.*;  
import java.awt.image.*;  
import java.util.*;
import javax.swing.*;
import javax.imageio.*;
import java.io.*;
import java.nio.ByteBuffer;

//==============================================
public class ImageMemoryOutputStream
{
   Image img;
   
   int remainingBytes = 0;
   
   int bitPos = 0;
   
   int[] pixels = null;
   
   //---------------------------------------------
   public ImageMemoryOutputStream(Image img) throws InterruptedException
   {
      this.img = img;
      
      int width = img.getWidth(null);
      int height = img.getHeight(null);
      
      remainingBytes = (width * height) * 3 / 8; 
      
      pixels = new int[width * height];
      
      try {
			PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
			pg.grabPixels();
		}
		catch(InterruptedException e) {
			throw e;
		}
   }
   
   //---------------------------------------------
   void write(int oneByte) throws IOException 
   {
      if(pixels == null) throw new IOException("ȼ ۰ ǵ ʾҽϴ.");
      
      if(remainingBytes == 0) throw new IOException("޸𸮰 մϴ");
        
      for(int i=7; i >= 0; i--) {
  
         int oneBit = (oneByte >> i) & 1;
         
         int pixelIndex = bitPos / 3;
            
         int rgbPos = (2 - (bitPos % 3)) * 8;
         
         pixels[pixelIndex] &= ~(1 << rgbPos);  
         
         if(oneBit == 1) pixels[pixelIndex] |= (1 << rgbPos);
           
         bitPos++;
      }
      
      remainingBytes--;
   }
   
   //---------------------------------------------
   void write(byte[] byteBuff) throws IOException 
   {
      if(byteBuff == null) throw new IOException("Byte buffer is null..");
      
      for(int i=0; i<byteBuff.length; i++) {
         write((int)byteBuff[i]);
      }
   }
   
   //---------------------------------------------
   void writeLong(long longVal) throws IOException 
   {
      write(longToBytes(longVal));
   }
   
   //---------------------------------------------
   void writeInt(int intVal) throws IOException 
   {
      write(intToBytes(intVal));
   }
   
   //---------------------------------------------
   void writeString(String s) throws IOException 
   {
      byte[] buff = s.getBytes();
      writeInt((int)(buff.length));
      write(buff);
   }
   
   //---------------------------------------------
   private byte[] longToBytes(long x) 
   {
      ByteBuffer buffer = ByteBuffer.allocate(Long.BYTES);
      buffer.putLong(x);
      return buffer.array();
   }
   
   //---------------------------------------------
   byte[] intToBytes(int x) 
   {
      ByteBuffer buffer = ByteBuffer.allocate(Integer.BYTES);
      buffer.putInt(x);
      return buffer.array();
   }
   
   //---------------------------------------------
   int[] getPixelArray()
   {
      return pixels;
   }

   //---------------------------------------------
   void close() throws IOException
   {
      write(-1);
      pixels = null;
      remainingBytes = 0;
      bitPos = 0;
   }
   
   //---------------------------------------------
   int available()
   {
      return remainingBytes;
   }
   
}